#!/bin/env python
# -*- coding: utf-8 -*-

import difflib
import sys
import argparse
import webbrowser
import os
import filecmp
import subprocess

from colorama import Fore, init, AnsiToWin32
init(wrap=False)
stream = AnsiToWin32(sys.stderr).stream

compResultFile = "comp_result.html"
notebookFile = "notebook.ipynb"

# 读取文件
def read_file(file_name):
    try:
        file_desc = open(file_name, 'r', encoding='utf-8')
        # 读取后按行分割
        text = file_desc.read().splitlines()
        file_desc.close()
        return text
    except IOError as error:
        print('Read input file Error: {0}'.format(error))
        sys.exit()

# 比较时忽略行末的空格和文件末尾的回车换行
def advanced_file_compare(file1, file2):
    f1 = open(file1, encoding='utf-8')
    f2 = open(file2, encoding='utf-8')
   
    returnVal = 1

    str1 = []
    for line1 in f1:
        line1 = line1.rstrip()
        line1 = line1.replace('\n', '')
        if len(line1) != 0:
            str1.append(line1)
    str2 = []
    for line2 in f2:
        line2 = line2.rstrip()
        line2 = line2.replace('\n', '')
        if len(line2) != 0:
            str2.append(line2)
    
    count = 0
    if len(str1) == len(str2):
        for line1 in str1:
            if line1 != str2[count]: #文件不同
                returnVal = 0
                break
            else:
                count = count + 1
    else:
        returnVal = 0
           
    f1.close()
    f2.close()
   
    return returnVal

# 比较两个文件并把结果生成一份html文本
def compare_file(file1, file2, seqNum):
    returnVal = 0
    if file1 == "" or file2 == "":
        print('文件路径不能为空：第一个文件的路径：{0}, 第二个文件的路径：{1} .'.format(file1, file2))
        sys.exit()
    else:
        print("正在将标准答案输出文件 {0} 与用户编写的应用程序输出文件 {1} 进行比较。\n比较结果".format(file1, file2), end=': ')
    
#    if os.path.isfile(file1) and os.path.isfile(file2) and filecmp.cmp(file1, file2):
    if os.path.isfile(file1) and os.path.isfile(file2) and advanced_file_compare(file1, file2):
        print("文件相同")
        print(Fore.GREEN + "Case{0} 验证成功".format(seqNum), file = stream, end='')
        print(Fore.WHITE, file = stream)
        returnVal = 1
        return returnVal
    else:
        print("文件不同")
        print(Fore.RED + "Case{0} 验证失败".format(seqNum), file = stream, end='')
        print(Fore.WHITE, file = stream)

    text1_lines = read_file(file1)
    text2_lines = read_file(file2)
    diff = difflib.HtmlDiff()    # 创建HtmlDiff 对象
    result = diff.make_file(text1_lines, text2_lines)  # 通过make_file 方法输出 html 格式的对比结果
    # 将结果写入到比较结果文件中
    try:
        with open(compResultFile, 'a+', encoding="utf-8") as result_file:
            promptContent = "<p>Case {0} 验证失败。</br>标准答案输出文件 output{0}.txt（左边）与用户编写的应用程序输出文件 user_output{0}.txt（右边）的比较结果：</p>".format(seqNum)
            result = promptContent + result
            result_file.write(result)
    except IOError as error:
        print('写入 html 文件错误：{0}'.format(error))
    finally:
        return returnVal


if __name__ == "__main__":
    print()
    print(Fore.YELLOW + "提示：\n1. 启用验证前一定要先保存 sql 文件。\n2. 每次启动验证，均会重新创建数据库、数据表和插入数据。", file = stream)
    print(Fore.WHITE, file = stream)
    
    # 移除比较结果文件
    if os.path.isfile(compResultFile):
        os.remove(compResultFile)

    seqNum = 1
    while 1:
        outputFile = "output{0}.txt".format(seqNum)
        useroutputFile = "user_output{0}.txt".format(seqNum)
        if seqNum == 1 and not os.path.isfile(outputFile):
            print(Fore.RED + "该项目未提供自动化验证功能", file = stream)
            exit(1)
        if not os.path.isfile(outputFile):
            break
        print("\n正在验证 case{0}。\n正在将 python 程序的输出内容保存到文件 {1} 中。".format(seqNum, useroutputFile))
        # 将对应的case号当参数传入
        runCommand = "python testcases.py local_test {0}".format(seqNum)
        execResult = os.system(runCommand)
        if execResult != 0:
            print(Fore.RED + "应用程序异常，返回值：{0}".format(execResult), file = stream)
            exit(1)
 
        if os.path.isfile(outputFile) and os.path.isfile(useroutputFile):
            if compare_file(outputFile, useroutputFile, seqNum) == 0:
                print("正在使用浏览器打开文件比较结果，可帮助用户查找验证失败的原因。")
                # 使用浏览器打开比较结果页面
                webbrowser.open('file://' + os.path.realpath(compResultFile))
                break
        seqNum = seqNum + 1
